﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Web;

using ICSharpCode.SharpZipLib.Zip;

using KheyaOrchardGallery.Models;

using Orchard;
using Orchard.Data;
using Orchard.FileSystems.Media;
using Orchard.Localization;
using Orchard.MediaLibrary.Models;
using Orchard.MediaLibrary.Services;
using Orchard.Validation;
using Orchard.UI.Notify;


namespace KheyaOrchardGallery.Services
{
	public class KheyaImageGalleryService : IKheyaImageGalleryService
	{
		private const string ImageGalleriesMediaFolder = "KheyaImageGalleries";
        
		private readonly IStorageProvider _storageProvider;
		private readonly IMediaLibraryService _mediaService;
        private readonly IList<string> _imageFileFormats = new[] { "BMP", "GIF", "EXIF", "JPG", "PNG", "TIFF" };
		private readonly IList<string> _fileFormats = new[] { "BMP", "GIF", "EXIF", "JPG", "PNG", "TIFF", "ZIP" };

		public KheyaImageGalleryService(IMediaLibraryService mediaService,IStorageProvider storageProvider)
		{
			_mediaService = mediaService;
			_storageProvider = storageProvider;

            if (_mediaService.GetMediaFolders(string.Empty).All(o => o.Name != ImageGalleriesMediaFolder))
			{
				_mediaService.CreateFolder(string.Empty, ImageGalleriesMediaFolder);
                _mediaService.CreateFolder(ImageGalleriesMediaFolder, "Kheya");
			}
		}
		public void CreateImageGallery(string name)
		{
			_mediaService.CreateFolder(ImageGalleriesMediaFolder, name);
		}
        
        public IEnumerable<string> AllowedFileFormats
		{
			get { return _fileFormats; }
		}
        public bool IsFileAllowed(string fileName, bool allowZip) {
			return (IsImageFile(fileName) || (allowZip && IsZipFile(Path.GetExtension(fileName))));
		}
		public bool IsFileAllowed(HttpPostedFileBase postedFile)
		{
			if (postedFile == null)
			{
				return false;
			}
			return IsFileAllowed(postedFile.FileName, true);
		}
        private bool IsImageFile(string fileName)
		{
			string extension = Path.GetExtension(fileName);
			if (extension == null)
				return false;
			extension = extension.TrimStart('.');

			return _imageFileFormats.Any(o => extension.Equals(o, StringComparison.OrdinalIgnoreCase));
		}
        private static bool IsZipFile(string extension)
		{
			return string.Equals(extension.TrimStart('.'), "zip", StringComparison.OrdinalIgnoreCase);
		}
        public void AddImage(string imageGalleryName, HttpPostedFileBase imageFile)
		{
			AddImage(imageGalleryName, imageFile.FileName, imageFile.InputStream);
		}

		public void AddImage(string imageGalleryName, string fileName, Stream imageFile)
		{
			if (!IsFileAllowed(fileName, true))
			{
				throw new InvalidOperationException(string.Format("{0} is not a valid file.", fileName));
			}

            if (IsZipFile(Path.GetExtension(fileName)))
			{
				
			}
			else
			{
				_mediaService.UploadMediaFile(GetMediaPath(imageGalleryName), fileName, imageFile);
			}
		}
        private string GetMediaPath(string imageGalleryName)
		{
			return _storageProvider.Combine(ImageGalleriesMediaFolder, imageGalleryName);
		}
        public Models.KheyaImageGallery GetImageGallery(string imageGalleryName)
		{
			var mediaFolder = _mediaService.GetMediaFolders(ImageGalleriesMediaFolder).SingleOrDefault(m => m.Name == imageGalleryName);

			if (mediaFolder != null)
			{
				return CreateImageGalleryFromMediaFolder(mediaFolder);
			}
			return null;
		}
        private string GetName(string mediaPath)
		{
			return mediaPath.Split(new[] { '\\', '/' }).Last();
		}
        private Models.KheyaImageGallery CreateImageGalleryFromMediaFolder(MediaFolder mediaFolder)
		{
			var images = _mediaService.GetMediaFiles(mediaFolder.MediaPath);
			
			return new Models.KheyaImageGallery
            {
                LastUpdated = mediaFolder.LastUpdated,
				MediaPath = mediaFolder.MediaPath,
				Name = mediaFolder.Name,
				Size = mediaFolder.Size,
				User = mediaFolder.User,
                Images = images.Select(image => CreateImageFromMediaFile(image))
            };
		}
        private KheyaImageGalleryImage CreateImageFromMediaFile(MediaFile mediaFile)
		{
			
			return new KheyaImageGalleryImage
			{
				PublicUrl = _mediaService.GetMediaPublicUrl(mediaFile.FolderName, mediaFile.Name),
				Name = mediaFile.Name,
				Size = mediaFile.Size,
				User = mediaFile.User,
				LastUpdated = mediaFile.LastUpdated
			};
		}
        public string GetMediaImageFolder(string imageGalleryName, string fileName)
		{
			var mediaFolder = _mediaService.GetMediaFolders(ImageGalleriesMediaFolder).SingleOrDefault(m => m.Name == imageGalleryName);

			if (mediaFolder != null)
			{
				return _mediaService.GetMediaPublicUrl(mediaFolder.MediaPath, fileName);
			}
			return null;
		}
        public void DeleteImage(string imageGalleryName, string imageName)
		{
            _mediaService.DeleteFile(GetMediaPath(imageGalleryName), imageName);
		}
	}
}